from dataclasses import dataclass


@dataclass
class ReducePath:
    path: list[bool]
    reasons: list[str]

    @classmethod
    def empty(cls):
        return cls([], [])

    def explore(self, choice):
        self.reasons = []
        self.path.append(choice)
        return self

    def left(self):
        return len(self.reasons) - len(self.path)

    def check(self, reason: str):
        self.reasons.append(reason)
        try:
            return self.path[len(self.reasons) - 1]
        except IndexError:
            return False


def reduce(predicate, reducer, i):
    r = ReducePath.empty()
    j = i
    ix = reducer(i, r.explore(True))
    # While we don't consume all choices going down the true branch
    while r.left() >= 0:
        print(r.reasons[len(r.path) - 1], "...", end="")
        if predicate(ix):
            # If true update the valid input
            print(" Yes")
            j = ix
        else:
            # If false we have to go down the left branch.
            print(" No")
            r.path[-1] = False
        ix = reducer(i, r.explore(True))
    return j


def debug(predicate):
    def newpred(i):
        t = predicate(i)
        print(" test", i, "...", end="")
        return t

    return newpred


def reduce_set(i: set, r) -> set:
    result: set = set()
    for e in sorted(i):
        if not r.check(f"remove {e}?"):
            result.add(e)
    return result


if __name__ == "__main__":
    reduce(debug(lambda a: "a" in a), reduce_set, {"a", "b", "c"})
